/**
 * The Node class represents one vertex in the n-dimensional cube
 */
public class Node
{
  protected Node next[]; // number of elements = dimension
  protected boolean digits[]; // number of elements = dimension
  protected Position pos;

  
  public void setPosition(Position p) { pos=p; }
  public Position getPosition()       { return pos; }
  public Node getNext(int i)          { return next[i]; }

  
  public Node(boolean[] parDigits)
  {
    digits=parDigits;
    pos=null;
    next=new Node[Main.dim];
    
    for (int i=0; i<Main.dim; i++) next[i]=null;
  } // Node()

  
  public boolean hasSameDigits(boolean[] parDigits)
  {
    //System.out.println("comparing this ("+boolArray2String(digits)+") with "+boolArray2String(parDigits));
    for (int i=0; i<Main.dim; i++)
    {
      if (digits[i]!=parDigits[i]) return false;
    }
    return true;
  } // hasSameDigits()

  
  public void makeChildren()
  // recursive
  {
    for (int i=0; i<Main.dim; i++)
    {
      if (!digits[i]) // if it is zero
      {
        boolean newDigits[]=new boolean[Main.dim];
        for (int j=0; j<Main.dim; j++)
        {
          if (j==i) newDigits[j]=true; // toggle it to one !
          else newDigits[j]=digits[j]; // else copy the old digits
        }
        Position newPos=null;
        Node child=Node.searchNode(newDigits);
        if (child==null) // node not found
        {
          //System.out.println("NewDigits "+boolArray2String(newDigits)+" not found");
          next[i]=new Node(newDigits);
          Position vec=AuxClass.getVector(i);
          newPos=AuxClass.buildCoord(vec,pos);
          next[i].setPosition(newPos);
//          AuxClass.drawLine(pos,newPos);
        } // if
        else
        {
          next[i]=child;
          newPos=child.getPosition();
        }
        //System.out.println("Drawing line from ["+boolArray2String(   digits)+"] "+pos+
        //                                 " to ["+boolArray2String(newDigits)+"] "+newPos);
        AuxClass.drawLine(pos,newPos);
        next[i].makeChildren();
      } // if
    } // for
  } // makeChildren()

  
  public String getDigitsAsString()
  {
    StringBuffer buf=new StringBuffer();
    
    for (int i=0; i<Main.dim; i++)
    {
      buf.append(digits[i]?"1":"0");
    } // for
    
    return buf.toString();
  } // getDigitsAsString()
  
  
  public String boolArray2String(boolean[] arr)
  {
    StringBuffer sb=new StringBuffer();
    
    for (int i=0; i<arr.length; i++)
    {
      sb.append(arr[i]?"1":"0");
    }
    
    return sb.toString();
  } // boolArray2String()


  public static Node searchNode(boolean[] digits)
  {
    return searchNodeRec(digits,Main.root);
  } // searchNode()

  
  public static Node searchNodeRec(boolean[] digits,Node node)
  {
    if (node==null) return null;
    
    if (node.hasSameDigits(digits)) return node;
    else
    {
      for (int i=0; i<Main.dim; i++)
      {
        Node result=searchNodeRec(digits,node.getNext(i));
        if (result!=null) return result;
      }
      return null;
    } // else
  } // searchNodeRec()

  
  public String toString()
  // recursive !
  {
    StringBuffer str=new StringBuffer();
    int i;
    
    str.append("---------------\n");
    str.append("dim="+Main.dim+"\n");
    str.append("digits="+getDigitsAsString()+"\n");
    
    // write pos
    
    for (i=0; i<Main.dim; i++)
    {
      str.append("next["+i+"]=");
      if (next[i]==null) str.append("null\n");
      else str.append("\n"+next[i]);
    } // for
    str.append("---------------\n");
    
    return str.toString();
  } // toString()
  
} // class Node